/*
 * Types and constants for processing images of FAT-formatted disks.
 */

/* Attribute bits in directory entries. */
#define FAT_ATTR_RDO        0x01
#define FAT_ATTR_HID        0x02
#define FAT_ATTR_SYS        0x04
#define FAT_ATTR_VOL        0x08
#define FAT_ATTR_DIR        0x10
#define FAT_ATTR_ARC        0x20

/* Attribute combination used by LFN directory entries. */
#define FAT_ATTR_LFN        (FAT_ATTR_VOL | FAT_ATTR_SYS | \
                             FAT_ATTR_HID | FAT_ATTR_RDO)

/* All horrible misaligned stuff. */
#pragma pack(1)

typedef struct {
    uint8_t     jump[3];
    char        oem_name[8];
    uint16_t    bytes_per_sect;
    uint8_t     sec_per_clust;
    uint16_t    reserved_sect;
    uint8_t     num_of_fats;
    uint16_t    root_entries;
    uint16_t    total_sect;
    uint8_t     media_desc;
    uint16_t    sect_per_fat;
    uint16_t    sect_per_trk;
    uint16_t    num_heads;
    uint32_t    hidden_sect;
    uint32_t    ext_total_sect;
    uint8_t     phys_drive_no;
    uint8_t     rsvd2;
    uint8_t     ext_signature;
    uint32_t    vol_id;
    char        vol_label[11];
    uint8_t     rsvd3[8];
} fat_boot_sec;


/* The original BPB defined by DOS 2.0. The last three fields are optional
 * and not technically part of the BPB; they do not describe the FAT file
 * system per se but rather help the disk device driver understand the
 * medium structure.
 */
typedef struct {
    uint16_t    bytes_per_sect; /* Bytes per sector. */
    uint8_t     sec_per_clust;  /* Sectors per cluster (power of 2). */
    uint16_t    reserved_sect;  /* Reserved sectors before first FAT.*/
    uint8_t     num_of_fats;    /* Number of FATs (almost always 2). */
    uint16_t    root_entries;   /* Number of root directory entries. */
    uint16_t    total_sect;     /* Total sectors on logical image. */
    uint8_t     media_desc;     /* Media descriptor byte. */
    uint16_t    sect_per_fat;   /* Sectors per FAT. */
    uint16_t    sect_per_trk;   /* Sectors per track (optional). */
    uint16_t    num_heads;      /* Number of heads (optional). */
    uint32_t    hidden_sect;    /* Hidden sectors (optional). */
} bpb_dos_20;


/* The extended BPB introduced in DOS 4.0. The extended BPB signature in
 * the boot sector must contain 29h for the extended BPB to be valid.
 */
typedef struct {
    uint16_t    bytes_per_sect; /* Bytes per sector. */
    uint8_t     sec_per_clust;  /* Sectors per cluster (power of 2). */
    uint16_t    reserved_sect;  /* Reserved sectors before first FAT.*/
    uint8_t     num_of_fats;    /* Number of FATs (almost always 2). */
    uint16_t    root_entries;   /* Number of root directory entries. */
    uint16_t    total_sect;     /* Total sectors on logical image. */
    uint8_t     media_desc;     /* Media descriptor byte. */
    uint16_t    sect_per_fat;   /* Sectors per FAT. */
    uint16_t    sect_per_trk;   /* Sectors per track (optional). */
    uint16_t    num_heads;      /* Number of heads (optional). */
    uint32_t    hidden_sect;    /* Hidden sectors (optional). */
    uint32_t    ext_total_sect; /* 32-bit total sectors. */
} bpb_dos_40;


typedef struct {
    char        name[8];
    char        ext[3];
    uint8_t     attr;
    char        rsvd[10];
    uint16_t    time;
    uint16_t    date;
    uint16_t    cluster;
    uint32_t    size;
} fat_dir_entry;

#pragma pack()

