/* Convert a floppy image in any of the supported formats to a raw image. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "img_defs.h"
#include "img_help.h"
#include "img_fns.h"

extern img_fns      ifn;

int main( int argc, char **argv )
{
    char        sect_buf[IMG_SEC_SZ_MAX];
    const char  *comment;
    img_handle  fsrc;
    int         sector;
    FILE        *fdst;
    int         rc;

    if( argc != 3 ) {
        printf( "usage: %s <src_img> <dst_raw_img>\n", argv[0] );
        return( EXIT_FAILURE );
    }

    /* If the image layer opens (for reading) the same file we open
     * here for writing, the source will get trashed.
     */
    if( !strcmp( argv[1], argv[2] ) ) {
        printf( "Source and destination files must differ!\n" );
        return( EXIT_FAILURE );
    }

    rc = img_open( argv[1], &ifn, &fsrc );
    if( rc ) {
        printf( "Failed to open `%s' for reading!\n", argv[1] );
        printf( "Error code: %d\n", rc );
        return( EXIT_FAILURE );
    }

    fdst = fopen( argv[2], "wb" );
    if( !fdst ) {
        printf( "Failed to open `%s' for writing!\n", argv[2] );
        img_close( fsrc );
        return( EXIT_FAILURE );
    }

    comment = img_query_comment( fsrc );
    if( comment )
        puts( comment );

    sector = 0;
    while( (rc = img_read_sect( fsrc, sector++, &sect_buf )) > 0 ) {
        fwrite( &sect_buf, 1, rc, fdst );
    }

    /* 'No such sector' indicates the end of an image when we're reading
     * sequentially. Anything else is a real error.
     */
    if( rc != IMG_ERR_NO_SECT ) {
        printf( "Image extraction failed at sector %d (%d)\n", sector, rc );
    }

    img_close( fsrc );
    fclose( fdst );
    return( EXIT_SUCCESS );
}
